/*
 * (C) COPYRIGHT 2012 CRZ
 *
 * File Name : bsharp_process.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 11/27/2012
 */

/* includes */

#include "hw_config.h"

/* global variables */

BSharp_Process_StateE g_bsharp_process_state;

unsigned char g_rx_packet_data[BSHARP_DATA_MAX_SIZE];
unsigned char g_typecode, g_opcode, g_byte_count;
unsigned char g_send_recv, g_ack_noack, g_data_type;

unsigned char g_send_packet[BSHARP_PACKET_SIZE];

int com_flag, send_flag;

char g_data_len;
char g_endian;
char g_interval;
char g_vref;
int  g_current = 0;
int  g_num;
char g_data_H, g_data_L;

unsigned char g_prescaler;

#define BSHARP_TIME_VOLT_MAX_POINT 80
char g_time[BSHARP_TIME_VOLT_MAX_POINT];
char g_volt[BSHARP_TIME_VOLT_MAX_POINT];

#define BSHARP_ADC_TYPE_VOLTAGE  0
#define BSHARP_ADC_TYPE_CURRENT  1

/* local functions */

static void bsharp_delay(u32 value)
{
    u32 delay_time;

    delay_time = g_prescaler * value;

    printf("Now Delay ... %d msec\n", delay_time);
    Delay(delay_time);
}

static void bsharp_current_average(void)
{
    int temp1, i;
    char ttt;

    DEBUG_MSG_FUNC_START;

    g_current = 0;

    for (i = 0 ; i < g_num ; i ++){
        temp1 = 0x0000;
        temp1 = g_volt[2*i];
        temp1 = temp1 << 8;
        temp1 = temp1 + g_volt[2*i + 1];
        g_current = g_current + temp1;
        ttt = g_current >> 8; //DEBUG

        printf("%s() g_current: %X, ttt: %X\n", __FUNCTION__, g_current, ttt);
    }
    g_current = g_current / g_num;
}

static void bsharp_ad_convert(unsigned int type)
{
    int ret;
    u16 adc_value;

    DEBUG_MSG_FUNC_START;

    ret = Get_AD7321_ADC_Data(type);
    if(ret < 0)
    {
        DEBUG_MSG_FUNC_ERROR;
    }
    adc_value = (u16)ret;
    
    g_data_H = (adc_value >> 8) & 0xFF;
	g_data_L = adc_value & 0xFF;
}

static void bsharp_dac_out(char data_H, char data_L)
{
    u16 dacVal = 0, dacValRx;

    dacVal = data_H << 8 | data_L;
    printf("%s() data_H: 0x%02X, data_L: 0x%02X, dacVal: 0x%04X\n", __FUNCTION__, data_H, data_L, dacVal);

    DAC101C08_Set_Dac_Value(dacVal);

    dacValRx = DAC101C08_Get_Dac_Value();

    printf("dacVal: 0x%X, dacValRx: 0x%X, half_dacValRx: 0x%X\n", dacVal, dacValRx, dacValRx/2);
}

static void bsharp_receive(void)
{
  int ret = 0;
  unsigned char packet[BSHARP_PACKET_SIZE];
  
  memset(packet, 0, BSHARP_PACKET_SIZE);
  memset(g_send_packet, 0, BSHARP_PACKET_SIZE);

  ret = SLIP_Receive_Packet(packet, BSHARP_PACKET_SIZE);
  printf("Received Packet Size: %d\n", ret);
  if(ret < 0)
  {
    DEBUG_MSG_FUNC_ERROR;
    return;
  }
  BSharp_Packet_Print(packet, ret);
  if(ret < (BSHARP_PREAMBLE_SIZE + BSHARP_COMMAND_SIZE))
  {
    DEBUG_MSG_FUNC_ERROR;
    printf("B# command data should be larger or equal size %d\n",
           BSHARP_PREAMBLE_SIZE + BSHARP_COMMAND_SIZE);
    return;
  }

  ret = BSharp_Packet_Check_Preamble(packet);
  if(ret < 0)
  {
    DEBUG_MSG_FUNC_ERROR;
    return;
  }

  // if received data is normal, change state with Normal ...
  g_bsharp_process_state = BSHARP_PROC_STATE_NORMAL;

  g_typecode    = packet[BSHARP_TYPECODE_OFFSET];
  g_opcode      = packet[BSHARP_OPCODE_OFFSET];
  g_byte_count = packet[BSHARP_COUNT_OFFSET];
  printf("typecode: 0x%02X, opcode: 0x%02X, data_length: 0x%02X\n",
         g_typecode, g_opcode, g_byte_count);

  BSharp_Packet_Get_Data(packet, g_byte_count);
}

static void bsharp_receive_process(void)
{
  int i;
  
  DEBUG_MSG_FUNC_START;
  bsharp_receive();
  
  g_send_recv = g_typecode & BSHARP_SEND_RECV_MASK;
  g_ack_noack = g_typecode & BSHARP_ACK_NOACK_MASK;
  g_data_type = g_typecode & BSHARP_DATA_TYPE_MASK;

  printf("%s() g_send_recv: 0x%02X, g_ack_noack: 0x%02X, g_data_type: 0x%02X\n", 
         __FUNCTION__, g_send_recv, g_ack_noack, g_data_type);

  if (g_send_recv == BSHARP_SEND)
  {
    if (g_data_type == BSHARP_MAX_TICK)
    {
        g_interval = g_rx_packet_data[g_endian];
        printf("%s() BSHARP_MAX_TICK g_interval: 0x%X\n", __FUNCTION__, g_interval);
    }

    if (g_data_type == BSHARP_VOLTAGE)
    {
      printf("%s() BSHARP_VOLTAGE: ", __FUNCTION__);
      for(i = 0; i < g_byte_count ; i ++)
      {
        g_volt[i] = g_rx_packet_data[i];
        printf("%02X", g_volt[i]);
      }
      printf("\n");
    }

    if (g_data_type == BSHARP_TIME)
    {
      printf("%s() BSHARP_TIME: 0x", __FUNCTION__);
      for(i = 0; i < g_byte_count; i ++)
      {
        g_time[i] = g_rx_packet_data[i];
        printf("%02X", g_time[i]);
      }
      printf("\n");
      g_data_len = g_byte_count;
    }						

    if (g_data_type == BSHARP_PRESCALAR)
    {
      g_prescaler = g_rx_packet_data[g_endian]; /* for PC */
      // g_prescaler = g_rx_packet_data[1]; /* for mac */
      printf("%s() BSHARP_PRESCALAR: 0x02%X\n", __FUNCTION__, g_prescaler);
    }

    if (g_data_type == BSHARP_ENDIAN)
    {
      g_endian = g_rx_packet_data[0];	/* for PC */
      printf("%s() BSHARP_ENDIAN: 0x%02X\n", __FUNCTION__, g_endian);
    }

    if (g_data_type == BSHARP_VREFADC_INDEX_MASK)
    {
      g_vref = g_rx_packet_data[0]; /* for PC */
      printf("%s() BSHARP_VREFADC_INDEX_MASK: 0x%02X\n", __FUNCTION__, g_vref);

      if (g_vref == BSHARP_VREF_4)
      {
        // adcon1 = 0x81;
        printf("Set the REF Voltage to 4.096V\n");
      }
      else if (g_vref == BSHARP_VREF_5)
      {
        // adcon1 = 0x80; // VDD is Ref, MSB 6 are 0
        printf("Set the REF Voltage to 5V\n");
      }
    }
  }
  else
  {
    printf("%s() type is NOT BSHARP_SEND\n", __FUNCTION__);
  }

  if (g_opcode == BSHARP_NOP)
  {
  }
  else
  {
    com_flag = 0;
    printf("com_flag is cleared.\n");
  }
  send_flag = 1;
  printf("send_flag is set.\n");
}

static void bsharp_send_packet_to_host(u8 cmd_and_data_only_size)
{
    printf("%s() data_size: 0x%02X\n", __FUNCTION__, cmd_and_data_only_size);
    SLIP_Send_Packet(g_send_packet, BSHARP_PREAMBLE_SIZE + cmd_and_data_only_size);
}

static void bsharp_send_process(void)
{
  DEBUG_MSG_FUNC_START;

  int i;
  char temp;

  printf("%s() g_send_recv: 0x%02X, g_ack_noack: 0x%02X, g_data_type: 0x%02X\n", 
       __FUNCTION__, g_send_recv, g_ack_noack, g_data_type);
    
  if (g_ack_noack == BSHARP_ACK)
  {
    g_send_packet[BSHARP_TYPECODE_OFFSET] = BSHARP_ACK; 
    g_send_packet[BSHARP_OPCODE_OFFSET]   = 0x00;
    g_send_packet[BSHARP_COUNT_OFFSET]    = 0x00;
    bsharp_send_packet_to_host(3);
    printf("ACK was Sent!\n");
  }

  if (g_send_recv == BSHARP_RECV)
  {
    printf("%s() BSHARP_RECV\n", __FUNCTION__);

    if (g_data_type == BSHARP_AVG_CURRENT)
    {
      printf("%s() BSHARP_AVG_CURRENT\n", __FUNCTION__);

      temp = g_current >> 8;
      printf("temp: %X, g_current: 0x%02X\n", temp, g_current);

      g_send_packet[BSHARP_TYPECODE_OFFSET] = BSHARP_AVG_CURRENT; 
      g_send_packet[BSHARP_OPCODE_OFFSET]   = 0x00;
      g_send_packet[BSHARP_COUNT_OFFSET]    = 0x02;

      if(g_endian == 0)
      {
        g_send_packet[BSHARP_DATA_OFFSET + 1] = g_current >> 8; /* for PC */
        g_send_packet[BSHARP_DATA_OFFSET + 0] = g_current ;
      }
      else
      {
        g_send_packet[BSHARP_DATA_OFFSET + 0] = g_current >> 8; /* for MAC */
        g_send_packet[BSHARP_DATA_OFFSET + 1] = g_current ;
      }
      bsharp_send_packet_to_host(5);

      printf("CURRENT_AVG was SENT!\n");
    }

    if (g_data_type == BSHARP_CURRENT)
    {
      printf("%s() BSHARP_CURRENT\n", __FUNCTION__);

      temp = 2; /* interval */
      g_send_packet[BSHARP_TYPECODE_OFFSET] = BSHARP_CURRENT; 
      g_send_packet[BSHARP_OPCODE_OFFSET]   = 0x00;
      g_send_packet[BSHARP_COUNT_OFFSET]    = temp;

      for (i = 0 ; i < temp ; i ++)
      {
          g_send_packet[BSHARP_DATA_OFFSET + i] = g_time[i];
      }
      bsharp_send_packet_to_host(3 + temp);

      printf("MEASURE_CURRENT was SENT!\n");
    }

    if (g_data_type == BSHARP_VOLTAGE)
    {
      temp = 2; /* interval */
      g_send_packet[BSHARP_TYPECODE_OFFSET] = BSHARP_VOLTAGE; 
      g_send_packet[BSHARP_OPCODE_OFFSET]   = 0x00;
      g_send_packet[BSHARP_COUNT_OFFSET]    = temp;

      for (i = 0 ; i < temp ; i ++)
      {
          g_send_packet[BSHARP_DATA_OFFSET + i] = g_volt[i];
      }
      bsharp_send_packet_to_host(3 + temp);

      printf("MEASURE_VOLTAGE was SENT!\n");
    }
  }
}

static void bsharp_work_process_emulate_voltage(void)
{
  char i, tmpVal, tmp_time;

  DEBUG_MSG_FUNC_START;

  g_num = g_data_len / 2;
  printf("g_num: %d, g_data_len: %d\n", g_num, g_data_len);

  for(i = 0; i < g_num; i ++)
  {
    printf("iteration %d start ...\n", i);
    if(1 == g_endian){ /* MAC */
      g_data_H = g_volt[2 * i];
      g_data_L = g_volt[2 * i + 1];
    }else{
      g_data_L = g_volt[2 * i];
      g_data_H = g_volt[2 * i + 1];
    }

    bsharp_dac_out(g_data_H, g_data_L);
    printf("Volt Output!!\n");

    if (BSHARP_MEASURE_AVG_CURRENT == (g_opcode & BSHARP_MEASURE_AVG_CURRENT)){
      bsharp_ad_convert(BSHARP_ADC_TYPE_CURRENT);
      g_volt[2 * i]     = g_data_H; 
      g_volt[2 * i + 1] = g_data_L;
      printf("Measure a Current Value. g_volt[%d]: %X, g_volt[%d]: %X\n",
             2 * i, g_data_H, 2 * i + 1, g_data_L);
    }

    tmpVal = 2 * (i + 1);
    if(tmpVal < g_data_len)
    {
      tmp_time = g_time[tmpVal + g_endian];
      bsharp_delay(tmp_time);
    }
    printf("TICK\n");
  }

  bsharp_current_average();
  printf("Emulation has finished!!\n");
}

static void bsharp_work_process(void)
{
    DEBUG_MSG_FUNC_START;
    
    if(BSHARP_EMULATE_VOLTAGE == (g_opcode & BSHARP_EMULATE_VOLTAGE)){
        bsharp_work_process_emulate_voltage();
    }

    // #define	BSHARP_MEASURE_VOLTAGE      '\x80'
    // #define	BSHARP_MEASURE_CURRENT      '\x40'
    if((BSHARP_MEASURE_VOLTAGE | BSHARP_MEASURE_CURRENT) == g_opcode){ /* 0xC0 */
        // change state with Measure Voltage & Current ...
        g_bsharp_process_state = BSHARP_PROC_STATE_MEASURE_VOLT_CURRENT;

        BSharp_Work_Process_Measure_Volt_Current();
    }	
}

/* functions */

void BSharp_Initialization(void)
{
    DEBUG_MSG_FUNC_START;

    CLCD_Init();
    CLCD_Write_String(0, "[[ B# Test ]]");
    CLCD_Write_String(1, "Battery Emulator");

    I2C_LowLevel_Init(I2C1);
    printf("I2C_LowLevel_Init() done\n");

    // SPI common initialization
    SPI_1_Init_common();
    printf("SPI_1_Init_common() done\n");

    SPI_1_Init_forAD7321();
    printf("SPI_1_Init_forAD7321() done\n");

    AD7321_HW_Init();
    printf("AD7321_HW_Init() done\n");

    // initialization of global variables ...
	g_interval  = 60;
	g_endian    = 0;
    g_prescaler = 4;

    // DAC Out init ...
	//bsharp_dac_out(0x01,0xfc);
	bsharp_dac_out(0x05,0x00);
}

void BSharp_Process(void)
{
  DEBUG_MSG_FUNC_START;

  com_flag = 1;
  send_flag = 0;
  printf("com_flag is set.\nsend_flag is cleared.\n");
  while (com_flag)
  {
    bsharp_receive_process();
    if (send_flag)
    {
      bsharp_send_process();
      send_flag = 0;
      printf("send_flag is cleared.\n");
    }
  }

  // process NOP opcode ...
//  if(BSHARP_NOP == g_opcode)
//  {
//    printf("Opcode is NOP\n");
//    return;
//  }

  bsharp_work_process();
  
  printf("\n");
}

void BSharp_Work_Process_Measure_Volt_Current(void)
{
    DEBUG_MSG_FUNC_START;
    printf("Voltage Measure & Measure CMD is executing!\n");

    g_send_packet[BSHARP_TYPECODE_OFFSET] = BSHARP_CURRENT; 
    g_send_packet[BSHARP_OPCODE_OFFSET]   = 0x00;
    g_send_packet[BSHARP_COUNT_OFFSET]    = 4;

    bsharp_ad_convert(BSHARP_ADC_TYPE_CURRENT);

    g_send_packet[BSHARP_DATA_OFFSET + 2] = g_data_L;
    g_send_packet[BSHARP_DATA_OFFSET + 3] = g_data_H;

    bsharp_ad_convert(BSHARP_ADC_TYPE_VOLTAGE);

    g_send_packet[BSHARP_DATA_OFFSET + 0] = g_data_L;
    g_send_packet[BSHARP_DATA_OFFSET + 1] = g_data_H;

    bsharp_send_packet_to_host(7);

    bsharp_delay(g_interval);

    printf("Measuring has finished!!\n");
}

