/*
 * clcd_lmc1623.c
 */

#include "hw_config.h"

#ifndef ABS
#define ABS(n)     (((n) < 0) ? -(n) : (n))
#endif

#define LINE1 0x80 // 1000 0000(1AAA AAAA) => 1-line DDRAM address 00H set
#define LINE2 0xC0 // 1100 0000(1AAA AAAA) => 2-line DDRAM address 40H set

#define CLCD_MAX_COLUMN_SIZE  16

static char string_data_line_1[50];
static char string_data_line_2[50];

/* Local functions */

void CLCD_Update(void);

static void clcd_write_1_byte(u8 data)
{
    if(data & 0x1)  GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA0_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA0_PIN);

    if(data & 0x2)  GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA1_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA1_PIN);

    if(data & 0x4)  GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA2_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA2_PIN);

    if(data & 0x8)  GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA3_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA3_PIN);

    if(data & 0x10) GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA4_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA4_PIN);

    if(data & 0x20) GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA5_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA5_PIN);

    if(data & 0x40) GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA6_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA6_PIN);

    if(data & 0x80) GPIO_SetBits(GPIO_LCD_DATA, GPIO_LCD_DATA7_PIN);
    else            GPIO_ResetBits(GPIO_LCD_DATA, GPIO_LCD_DATA7_PIN);
}

static void clcd_write_control(u8 w_data)
{
    LMC1623_LCD_DISABLE();
    LMC1623_LCD_RS_CONTROL();
    LMC1623_LCD_WRITE();

    Delay(1);
    clcd_write_1_byte(w_data);
    Delay(1);

    LMC1623_LCD_ENABLE();
    Delay(1);
    LMC1623_LCD_DISABLE();

    Delay(2);
}

static void clcd_write_data(u8 w_data)
{
    LMC1623_LCD_DISABLE();
    LMC1623_LCD_RS_DATA();
    LMC1623_LCD_WRITE();

    Delay(1);
    if(0 == w_data)
        clcd_write_1_byte(' ');
    else
        clcd_write_1_byte(w_data);
    Delay(1);

    LMC1623_LCD_ENABLE();
    Delay(1);
    LMC1623_LCD_DISABLE();

    Delay(2);
}

//*****************************************************************************
// Function:        void reverse(char *s)
// Input:           A string
// Output:          None
// Overview:        reverse string s in place
// Note:            None
//*****************************************************************************
static void clcd_reverse(char *s)
{
    unsigned char i, j;
    char c;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--)
    {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

//*****************************************************************************
// Function:        void itoa(int n, char *s)
// Input:           An Integer.
// Output:          A character string.
// Overview:        convert integer n to characters in s.
// Note:            None
//***************************************************************************** 
static u8 clcd_itoa(int n, char * str_p)
{
    int sign;
    unsigned char i;

    if ((sign = n) < 0)  		// record sign
        n = -n;          		// make n positive
    i = 0;
    do 
	{       					// generate digits in reverse order
        str_p[i++] = n % 10 + '0';  // get next digit
    } 
	while ((n /= 10) > 0);     	// delete it
    if (sign < 0)
        str_p[i++] = '-';
    str_p[i] = '\0';
    clcd_reverse(str_p);

    return i;
}

static u8 clcd_ftoa(long double f, char * str_p)
{
    signed char a;
    char size_int, size_float;

    size_int = clcd_itoa((int)f, str_p);
    printf("%s() size_int: %d\n", __FUNCTION__, size_int);
    
    str_p[size_int] = '.';
    size_int ++;
    
    a = ((char)(ABS(100*(f - (int)f)) + 0.5));
    printf("%s() a: %d\n", __FUNCTION__, a);
    size_float = clcd_itoa(a, &str_p[size_int]);
    printf("%s() size_float: %d\n", __FUNCTION__, size_float);

    return (size_int + size_float);
}

/* Global functions */

void CLCD_Write_Num(u8 lineNum, int n)
{
    char str[16];
    char size;

    CLCD_ClearLine(lineNum);
    
    size = clcd_itoa(n, str);
    printf("%s() size: %d\n", __FUNCTION__, size);
    CLCD_Write_String(lineNum, str);
}

void CLCD_Write_Float(u8 lineNum, long double f)
{
    char str[16] = {0};
    char size;
    
    CLCD_ClearLine(lineNum);
    
    size = clcd_ftoa(f, str);
    printf("%s() size: %d\n", __FUNCTION__, size);
    CLCD_Write_String(lineNum, str);
}

void CLCD_ClearLine(u8 lineNum)
{
    char str[16] = "                ";
    CLCD_Write_String(lineNum, str);
}

void CLCD_Write_String(u8 lineNum, char * str_p)
{
    u8 ch_len = strlen(str_p);

    if(ch_len > CLCD_MAX_COLUMN_SIZE)
        ch_len = CLCD_MAX_COLUMN_SIZE;

    if(lineNum == 0)
        strncpy(string_data_line_1, str_p, CLCD_MAX_COLUMN_SIZE);
    else
        strncpy(string_data_line_2, str_p, CLCD_MAX_COLUMN_SIZE);

    CLCD_Update();
}

void CLCD_Write_String_wFormat(u8 lineNum, const char * format_str_p, ...)
{
    char str[CLCD_MAX_COLUMN_SIZE];
    va_list argList;

    sprintf(str, format_str_p, argList);

    CLCD_Write_String(lineNum, str);
}

void CLCD_Update(void)
{
    u16 i;
    char * str1 = string_data_line_1;
    char * str2 = string_data_line_2;

    clcd_write_control(LINE1);
    for(i = 0; i < CLCD_MAX_COLUMN_SIZE; i ++)
    {
        clcd_write_data(*str1++);
    }

    clcd_write_control(LINE2);
    for(i = 0; i < CLCD_MAX_COLUMN_SIZE; i ++)
    {
        clcd_write_data(*str2++);
    }
}

void CLCD_Init(void)
{
    #define CLCD_FUNCSET_8BIT   0x38

    clcd_write_control(CLCD_FUNCSET_8BIT);
    clcd_write_control(0x0C);
    clcd_write_control(0x01);
    clcd_write_control(0x06);
}

