//*****************************************************************************
//! @file       cc112x_rx_sniff_mode_rx_term_on_cs.c
//! @brief      
//!
//! Revised     $Date: 2013-04-11 20:13:51 +0200 (to, 11 apr 2013) $
//! Revision    $Revision: 9716 $
//
//  Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
//
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//****************************************************************************/


/*****************************************************************************
* INCLUDES
*/
#include "cc112x_spi.h"
#include "stdlib.h"
#include "cc112x_rx_sniff_mode_reg_config.h"
#include "cc1120_test.h"

/******************************************************************************
* CONSTANTS
*/ 

/******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED 1
#define ISR_IDLE            0

/******************************************************************************
* LOCAL VARIABLES
*/
static uint32 packetCounter;

/******************************************************************************
* STATIC FUNCTIONS
*/
static void registerConfig(void);
static void manualCalibration(void);
static void calibrateRCOsc(void);
static void runWOR(void);
//static void radioRXISR(void);
static void updateUart(uint8 dataArray[]);

void printString(uint8 *string);
void printInt(int32 number);
uint8 getLengthInt(int32 number);

/******************************************************************************
* @fn          main
*
* @brief       Runs the main routine
*                
* @param       none
*
* @return      none
*/
void rx_sniff_mode_term_on_cs_rx(void)
{
  // initialize MCU and peripherals
//  initMCU();
  
  // write radio registers
  registerConfig();
  
  /***********************************************/
  /* TEST UART */
//  uint8 a = 50;

  printf("RX Sniff Mode Test\n");
  
  /**********************************************/
  
  // run rx sniff mode
  runWOR();
}
/******************************************************************************
* @fn          runWOR
*
* @brief       tbd
*              
*                
* @param       none
*
* @return      none
*/
static void runWOR(void)
{
  uint8 rxBuffer[128];
  uint8 rxBytes;
//  uint8 writeByte;

/*  
  // Connect ISR function to GPIO2
  ioPinIntRegister(IO_PIN_PORT_1, 8, &radioRXISR);
  // Interrupt on falling edge
  ioPinIntTypeSet(IO_PIN_PORT_1, 8, IO_PIN_FALLING_EDGE);
  // Enable interrupt
  ioPinIntEnable(IO_PIN_PORT_1, 8);
*/

  manualCalibration();
  
  // update LCD
//  updateLcd();
  
  // calibrate RC Osc
  calibrateRCOsc();
  
  // set radio in WOR
  trxSpiCmdStrobe(CC112X_SWOR);
  
  // reset packet counter
  packetCounter = 0;
  
  // infinite loop
  while(1)
  {
    // Wait for packet received interrupt 
    if(packetSemaphore == ISR_ACTION_REQUIRED)
    {
      // Read number of bytes in rx fifo
      cc112xSpiReadReg(CC112X_NUM_RXBYTES, &rxBytes, 1);
      // check that we have bytes in fifo
      if(rxBytes != 0)
      {
        // Check for RXFIFO_ERROR (over/under flow situation) 
        if((trxSpiCmdStrobe(CC112X_SNOP) & 0xF0) == 0x60)
        {
          // RXFIFO ERROR - flush rx fifo
          trxSpiCmdStrobe(CC112X_SFRX);
        }
        else
        {
          // Read n bytes from rx fifo
          cc112xSpiReadRxFifo(rxBuffer, rxBytes);  
          // Check CRC ok (CRC_OK: bit7 in second status byte)
          // This assumes status bytes are appended in RX_FIFO
          // (PKT_CFG1.APPEND_STATUS = 1.)
          // If CRC is disabled the CRC_OK field will read 1
          if(rxBuffer[rxBytes-1] & 0x80)
          {
            // Update packet counter
            packetCounter++;
            // Update LCD
            //updateLcd();
            //update UART
            updateUart(rxBuffer);
          }
        }
      }      
      // Reset packet semaphore
      packetSemaphore = ISR_IDLE;
      // Set radio back in RX
      trxSpiCmdStrobe(CC112X_SWOR);
    }
  } 
}
/*******************************************************************************
* @fn          updateUart
*
* @brief       none
*
* @param       none
*
* @return      none
*/
void updateUart(uint8 dataArray[]){
//  uint8 asciiTable[] = {48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 65, 66, 67, 68, 69, 70};
//  char a[2];
//  uint8 i;
//  uint8 length = dataArray[0];
  //for (k = 0; k < 6; k++) {
  
  printf("Received OK: %d\n", packetCounter);
  
//  for (uint8 k = 0; k < length+1; k++) {   
//    i = dataArray[k];
//    a[0] = asciiTable[((i >> 4) & 0x0F)];
//    a[1] = asciiTable[(i & 0x0F)];
    //write to uart
   // printString("0x");printString((uint8*)a);printString(" ");

//  }
  // printString("\r\n ");
}
/*******************************************************************************
* @fn          radioRXISR
*
* @brief       ISR for packet handling in RX. Sets packet semaphore, puts radio
*              in idle state and clears isr flag.
*
* @param       none
*
* @return      none
*/
#if 0
static void radioRXISR(void) {
  
  // set packet semaphore
  packetSemaphore = ISR_ACTION_REQUIRED;
  // Clear isr flag
//  ioPinIntClear(IO_PIN_PORT_1, 8);
}
#endif

/******************************************************************************
 * @fn          initMCU
 *
 * @brief       Initialize MCU and board peripherals
 *                
 * @param       input, output parameters
 *
 * @return      describe return value, if any
 */
#if 0
static void initMCU(void){
  
  // Init clocks and I/O 
  bspInit(BSP_SYS_CLK_8MHZ);
  
  // Init leds 
  bspLedInit(); 

  // Init Buttons
  bspKeyInit(BSP_KEY_MODE_POLL);
  
  // Initialize SPI interface to LCD (shared with SPI flash)
  bspIoSpiInit(BSP_FLASH_LCD_SPI, BSP_FLASH_LCD_SPI_SPD);  
  
  // Init LCD
  lcdInit();

  // Instantiate tranceiver RF spi interface to SCLK ~ 4 MHz */
  //input clockDivider - SMCLK/clockDivider gives SCLK frequency
  trxRfSpiInterfaceInit(0x02);

  // Init UART to use with virtual COM port
  P5DIR |= 0x80;
  P5SEL |= 0xC0;
  
  UCA1CTL1 |= UCSWRST;
  
  UCA1CTL1 |= UCSSEL_2;   // SMCLK

  //UCA1BR0 = 0x23;       //16 MHz 57600 baud
  //UCA1BR1 = 0x01;
  
  UCA1BR0 = 0x8A;                     // 8 MHz 57600
  UCA1BR1 = 0x00;                     // 8 MHz 57600

  UCA1MCTL |= UCBRS_1 + UCBRF_0;

  UCA1CTL1 &= ~UCSWRST;
  
  // Enable global interrupt
  _BIS_SR(GIE);
}
#endif

/*******************************************************************************
* @fn          registerConfig
*
* @brief       Write register settings as given by SmartRF Studio
*
* @param       none
*
* @return      none
*/
static void registerConfig(void) {
  uint8 writeByte;
  // reset radio
  trxSpiCmdStrobe(CC112X_SRES);
  // write registers to radio
  for(uint16 i = 0; i < (sizeof  preferredSettingsTerminateCS/sizeof(registerSetting_t)); i++) {
    writeByte =  preferredSettingsTerminateCS[i].data;
    cc112xSpiWriteReg( preferredSettingsTerminateCS[i].addr, &writeByte, 1);
  }
}
/******************************************************************************
* @fn          updateLcd
*
* @brief       updates LCD buffer and sends bufer to LCD.
*                
* @param       none
*
* @return      none
*/
#if 0
static void updateLcd(void)
{
  // update screen
  lcdBufferClear(0);
  lcdBufferPrintString(0,"    RX Sniff Mode      ",0,eLcdPage0);
  lcdBufferSetHLine(0,0,LCD_COLS-1,eLcdPage7); 
  lcdBufferPrintString(0,"Received ok:",0,eLcdPage3);
  lcdBufferPrintInt(0,packetCounter,70,eLcdPage4);
  lcdBufferPrintString(0,"      Packet RX        ",0,eLcdPage7);
  lcdBufferSetHLine(0,0,LCD_COLS-1,55);
  lcdBufferInvertPage(0,0,LCD_COLS,eLcdPage7);
  lcdSendBuffer(0);
}
#endif

/******************************************************************************
* @fn          calibrateRcOsc
*
* @brief       calibrates the RC Oclillator used for the cc112x wake on radio 
*              functionality.
*                
* @param       none
*
* @return      none
*/
static void calibrateRCOsc(void)
{
  uint8 writeByte; 
  // read current register value
  cc112xSpiReadReg(CC112X_WOR_CFG0,&writeByte,1);
  // mask register bitfields and write new values
  writeByte = (writeByte & 0xF9) | (0x02 << 1);
  //write new register value
  cc112xSpiWriteReg(CC112X_WOR_CFG0,&writeByte,1);
  // strobe idle to calibrate RC osc
  trxSpiCmdStrobe(CC112X_SIDLE);
  //disable RC calibration
  writeByte = (writeByte & 0xF9) | (0x00 << 1);
  cc112xSpiWriteReg(CC112X_WOR_CFG0, & writeByte, 1); 
}
/*******************************************************************************
* @fn          printString
*
* @brief       Print a string to the UART
*
* @param       Pointer to string to be printed
*
* @return      none
*/
#if 0
void printString(uint8 *string) {
  uint8 i = 0;
  do {
    UCA1TXBUF = string[i];
    while(!(UCA1IFG&UCTXIFG));    
  } while (string[i++] != 0);
}
#endif
/******************************************************************************
 * @fn          printInt
 *
 * @brief       
 *                
 * @param       
 *
 * @return      
*/
void printInt(int32 number) {
  uint8 asciiString[11] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  uint8 j = 1;
  
  if(number < 0){
    asciiString[0] = '-';
    number *= (-1);
    j = 0;
  }
  
  int8 numOfDigits = getLengthInt(number);

  for(int8 i = (numOfDigits - j); i >= (1 - j); i--) {
    int32 temp = number/10;
    int32 digit = number - temp * 10;
    asciiString[i] = (digit + '0');  
    number = temp;
  }
  printString(asciiString);
}
                        
uint8 getLengthInt(int32 number) {
  if(number == 0)
    return 1;
  
  uint8 numOfDigits = 0;
  if(number < 0) {
    number *= (-1);
    numOfDigits++;
  }
  while(number >= 1) {
    number /= 10;
    numOfDigits++;
  }
  return numOfDigits;
}  
/******************************************************************************
* @fn          manualCalibration
*
* @brief       calibrates radio according to CC112x errata
*                
* @param       none
*
* @return      none
*/
#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2
static void manualCalibration(void) 
{
  uint8 original_fs_cal2;
  uint8 calResults_for_vcdac_start_high[3];
  uint8 calResults_for_vcdac_start_mid[3];
  uint8 marcstate;
  uint8 writeByte;
  
  // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
  
  // 2) Start with high VCDAC (original VCDAC_START + 2):
  cc112xSpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
  writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
  
  // 3) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
  
  do 
  {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
  
  // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with high VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);
  
  // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
  
  // 6) Continue with mid VCDAC (original VCDAC_START):
  writeByte = original_fs_cal2;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
  
  // 7) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
  
  do 
  {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
  
  // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with mid VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);
  
  // 9) Write back highest FS_VCO2 and corresponding FS_VCO and FS_CHP result
  if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] > calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) 
  {
    writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  }
  else 
  {
    writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  }
}
