/*
 * zigbee_test.c
 */

/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
#include "cc1120_mango_porting.h"
#include "cc1120_test.h"
#include "string.h"
#include "debug.h"
#include "cc112x_spi.h"
#include "stdlib.h"
#include "cc112x_easy_link_reg_config.h"

/*******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED 1

/******************************************************************************
* LOCAL VARIABLES
*/
uint8  packetSemaphore;

/***********************************************************************************
 * CONSTANTS
 */

GPIO_TypeDef *  g_cc1120_gpio_ncs;
SPI_TypeDef *   g_cc1120_spi_num_p;

uint16_t  g_cc1120_gpio_pin_reset, g_cc1120_gpio_pin_ncs;
uint16_t  g_cc1120_gpio_pin_0, g_cc1120_gpio_pin_1,
    g_cc1120_gpio_pin_2, g_cc1120_gpio_pin_3, g_cc1120_gpio_pin_4, g_cc1120_gpio_pin_5;

uint8_t   g_cc1120_gpio_pin_5_pin_source, g_cc1120_gpio_pin_5_irq;
uint32_t  g_cc1120_gpio_pin_5_exti_line = EXTI_Line5;

uint8_t   g_cc1120_gpio_pin_6_pin_source, g_cc1120_gpio_pin_6_irq;
uint32_t  g_cc1120_gpio_pin_6_exti_line = EXTI_Line6;

void cc112xRunTX(void);
void cc112xRunRX(void);

/*******************************************************************************
* @fn          CC1120_radioRxTxISR
* @brief       ISR for packet handling. Sets packet semaphore 
* @param       none
* @return      none
*/
void CC1120_radioRxTxISR(void)
{
    // Set packet semaphore
    packetSemaphore = ISR_ACTION_REQUIRED;  
}

void CC1120_Test(SPI_TypeDef* SPIx_p)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    u8 ch;
    uint8 readData;

    DEBUG_MSG_FUNC_START;

    g_cc1120_spi_num_p = SPIx_p;

    if(SPI1 == SPIx_p)
    {
        g_cc1120_gpio_ncs         = GPIO_SPI_1;
        g_cc1120_gpio_pin_ncs     = GPIO_SPI_1_NCS_PIN;
        g_cc1120_gpio_pin_reset   = GPIO_SPI_IO_PIN_7;

        g_cc1120_gpio_pin_0       = GPIO_SPI_IO_PIN_0;
        g_cc1120_gpio_pin_1       = GPIO_SPI_IO_PIN_1;
        g_cc1120_gpio_pin_2       = GPIO_SPI_IO_PIN_2;
        g_cc1120_gpio_pin_3       = GPIO_SPI_IO_PIN_3;
        g_cc1120_gpio_pin_4       = GPIO_SPI_IO_PIN_4;
        g_cc1120_gpio_pin_5       = GPIO_SPI_IO_PIN_5;

        g_cc1120_gpio_pin_5_irq        = EXTI9_5_IRQn;
        g_cc1120_gpio_pin_5_pin_source = GPIO_PinSource5;
        g_cc1120_gpio_pin_5_exti_line  = EXTI_Line5;

        g_cc1120_gpio_pin_6_irq        = EXTI9_5_IRQn;
        g_cc1120_gpio_pin_6_pin_source = GPIO_PinSource6;
        g_cc1120_gpio_pin_6_exti_line  = EXTI_Line6;
    }
    else
    {
        g_cc1120_gpio_ncs         = GPIO_SPI_2;
        g_cc1120_gpio_pin_ncs     = GPIO_SPI_2_NCS_PIN;
        g_cc1120_gpio_pin_reset   = GPIO_SPI_IO_PIN_15;

        g_cc1120_gpio_pin_0       = GPIO_SPI_IO_PIN_8;
        g_cc1120_gpio_pin_1       = GPIO_SPI_IO_PIN_9;
        g_cc1120_gpio_pin_2       = GPIO_SPI_IO_PIN_10;
        g_cc1120_gpio_pin_3       = GPIO_SPI_IO_PIN_11;
        g_cc1120_gpio_pin_4       = GPIO_SPI_IO_PIN_12;
        g_cc1120_gpio_pin_5       = GPIO_SPI_IO_PIN_13;

        g_cc1120_gpio_pin_5_irq        = EXTI15_10_IRQn;
        g_cc1120_gpio_pin_5_pin_source = GPIO_PinSource13;
        g_cc1120_gpio_pin_5_exti_line  = EXTI_Line13;

        g_cc1120_gpio_pin_6_irq        = EXTI15_10_IRQn;
        g_cc1120_gpio_pin_6_pin_source = GPIO_PinSource14;
        g_cc1120_gpio_pin_6_exti_line  = EXTI_Line14;
    }

    CC1120_GPIO_0_Interrupt_Setting(ENABLE);
//    CC1120_GPIO_2_Interrupt_Setting(ENABLE);

    /* Configure gpio as input : Key */
    GPIO_InitStructure.GPIO_Pin   = g_cc1120_gpio_pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    /* Configure pins: RESETn ---------------------------------*/
    GPIO_InitStructure.GPIO_Pin   = g_cc1120_gpio_pin_reset;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIO_SPI_IO, &GPIO_InitStructure);

    // IO7 pin sould be Low
    GPIO_SetBits(GPIO_SPI_IO, g_cc1120_gpio_pin_reset);
    Delay(100);
    GPIO_ResetBits(GPIO_SPI_IO, g_cc1120_gpio_pin_reset);
    Delay(100);

    // Write radio registers
//    registerConfig();

    Delay(2);

    // Read data
    cc112xSpiReadReg(CC112X_PARTNUMBER, &readData, 1);
    printf("CC1120 part num: 0x%02X\n", readData);

    cc112xSpiReadReg(CC112X_PARTVERSION, &readData, 1);
    printf("CC1120 part ver: 0x%02X\n", readData);

    while(1)
    {
        printf("\n\n------------------------\n");
        printf("(CC1120 Test) Press menu key\n");
        printf("------------------------\n");
        printf("1> Run Tx\n");
        printf("2> Run Rx\n");
        printf("------------------------\n");
        printf("x> quit\n\n");

#if 1
#if 0
        ch = USART_GetCharacter(USART1);
#else
        ch = UartSerialGetChar_Wait();
#endif
#else
        ch = '4';
#endif

        printf("%c is selected\n", ch);

        switch((char)ch)
        {
        case '1':
            // Enter runTX, never coming back
            cc112xRunTX();
            break;

        case '2':
            // Enter runRX, never coming back
            cc112xRunRX();
            break;
        }

        if('x' == (char)ch)
        {
            break;
        }
    }
}

/******************************************************************************
 * @fn          manualCalibration
 *
 * @brief       calibrates radio according to CC112x errata
 *                
 * @param       none
 *
 * @return      none
 */
#if 0

#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2

static void manualCalibration(void)
{
  uint8 original_fs_cal2;
  uint8 calResults_for_vcdac_start_high[3];
  uint8 calResults_for_vcdac_start_mid[3];
  uint8 marcstate;
  uint8 writeByte;
    
  // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 2) Start with high VCDAC (original VCDAC_START + 2):
  cc112xSpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
  writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 3) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
    
  // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with high VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);
    
  // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 6) Continue with mid VCDAC (original VCDAC_START):
  writeByte = original_fs_cal2;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 7) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);

  // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with mid VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);
    
  // 9) Write back highest FS_VCO2 and corresponding FS_VCO and FS_CHP result
  if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] > calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
    writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  } else {
    writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  }
}
#endif

