/*
 * (C) COPYRIGHT 2009 CRZ
 *
 * File Name : main.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 08/12/2009
 */

/* includes */

#include "hw_config.h"

/* global variables */

RCC_ClocksTypeDef  rcc_clocks;

bool g_TestProcessState = FALSE;

TestMode_E  g_test_mode = TEST_MODE_CC2520_TEST;

/* functions */

void System_Information()
{
    printf("ETRI NOS Platform Test ... %s %s\n\n", __DATE__, __TIME__);

    printf("SYSCLK_Frequency = %d\n",rcc_clocks.SYSCLK_Frequency );
    printf("HCLK_Frequency = %d\n",rcc_clocks.HCLK_Frequency );
    printf("PCLK1_Frequency = %d\n",rcc_clocks.PCLK1_Frequency );
    printf("PCLK2_Frequency = %d\n",rcc_clocks.PCLK2_Frequency );
    printf("ADCCLK_Frequency = %d\n",rcc_clocks.ADCCLK_Frequency );
}

/**
  * @brief  Configures the used Timers.
  * @param  None
  * @retval None
  */
void TIM_Configuration(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

    /* TIM2 configuration */
    TIM_TimeBaseStructure.TIM_Period = 0x4AF;          
    TIM_TimeBaseStructure.TIM_Prescaler = 0xEA5F;       
    TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  
    TIM_TimeBaseStructure.TIM_RepetitionCounter = 0x0000;
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

    /* TIM2 enable counter */
    TIM_Cmd(TIM2, ENABLE);

    /* Immediate load of TIM2 Precaler value */
    TIM_PrescalerConfig(TIM2, 0xEA5F, TIM_PSCReloadMode_Immediate);

    /* Clear TIM2 update pending flag */
    TIM_ClearFlag(TIM2, TIM_FLAG_Update);

    /* Enable TIM2 Update interrupt */
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
}

void USB_Test_Start (void);
void TIM_Configuration(void);

bool g_print_menu_control_flag = TRUE;

// Second Timer consume flag
bool g_sec_time_consume_flag = FALSE;

// Button 2, 3 Control flag
bool g_button_up_press_flag = FALSE;
bool g_button_down_press_flag = FALSE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("0> System Information\n");
        printf("---------------------\n");
        printf("1> LED Test\n");
        printf("2> KEY Test\n");
        printf("3> Connector 1 - CC2520 Test\n");
        printf("4> Connector 2 - CC2520 Test\n");
        printf("5> Connector 1 - CC1120 Test\n");
        printf("6> Connector 2 - CC1120 Test\n");
        printf("7> Connector 1 - ENC28J60 Test\n");
        printf("8> Connector 2 - ENC28J60 Test\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

void LedControl_forProgramStart(void)
{
#define  LED_CTRL_START_DELAY  30

    LED_On_All();
    Delay(LED_CTRL_START_DELAY);
    LED_Blink_All(5, LED_CTRL_START_DELAY);
    Delay(LED_CTRL_START_DELAY);
    LED_Off_All();
}

/*
 * Name   : main
 * Input  : None
 * Output : None
 * Return : None
 */
int main(void)
{
    uint8_t ch;

    /* System Clocks Configuration */
    RCC_Configuration();

    RCC_GetClocksFreq(&rcc_clocks);

    /* NVIC configuration */
    NVIC_Configuration();

    /* Configure the GPIO ports */
    GPIO_Configuration();

    /* EXTI configuration */
    EXTI_Configuration();

    /* UART initialization */
    USART1_Init_forSerial();
//    USART2_Init_forComm();

    printf("ETRI NOS Platform Test ... %s %s\n", __DATE__, __TIME__);

    /* Configure TIM */
    TIM_Configuration();

    /* Configure SPI */
    SPI_Configuration();

    /* Setup SysTick Timer for 1 msec interrupts  */
    if (SysTick_Config(rcc_clocks.SYSCLK_Frequency / 1000))
    { 
        /* Capture error */ 
        while (1);
    }

    CLCD_Init();

    CLCD_Write_String_wFormat(0, "ETRI NOS System");
    CLCD_Write_String_wFormat(1, "%s\n", __DATE__);

    LedControl_forProgramStart();
    
    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '0':
                System_Information();
                break;

            case '1':
                LED_Test();
                break;

            case '2':
                KEY_Test();
                break;

            case '3':
                g_test_mode = TEST_MODE_CC2520_TEST;
                ZigBee_Test(SPI1);
                break;

            case '4':
                g_test_mode = TEST_MODE_CC2520_TEST;
                ZigBee_Test(SPI2);
                break;

            case '5':
                g_test_mode = TEST_MODE_CC1120_TEST;
                CC1120_Test(SPI1);
                break;

            case '6':
                g_test_mode = TEST_MODE_CC1120_TEST;
                CC1120_Test(SPI2);
                break;

            case '7':
                g_test_mode = TEST_MODE_ENC28J60_TEST;
                ENC28J60_Test(SPI1);
                break;

            case '8':
                g_test_mode = TEST_MODE_ENC28J60_TEST;
                ENC28J60_Test(SPI2);
                break;
            }

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }

    return 0;
}
